function [e n] = convert_to_utm(k, l, proj, zn)

% converts latitude (k, in degrees) and longitude (l, in degrees)
% to e and n (x and y) coordinates in a transverse Mercator
% projection. The variable 'proj' indicates the particular
% projection. If it is a UTM-type projection, the variable zn
% specifies the zone number. For Northern hemisphere zones, zn is
% a positive integer between 1 and 60. For Southern hemisphere zones,
% zn is a negative integer between -1 and -60. In either case, the longitude
% of the central meridian is 6*abs(zn)-183 degrees.
%
% More projections can be implemented using the 'switch' statement.
%
% Latitudes in the Southern hemisphere are negative.
% Longitudes in the Western hemisphere are negative.
%
% The parameters of the projection are
%   a: equatorial radius of ellipsoid
%   b: polar radius of ellipsoid
%   f0: scale factor on central meridian
%   k0: latitude of origin (radians)
%   l0: longitude of central meridian (radians)
%   e0: eastings coordinate of origin
%   n0: northings coordinate of meridian
%
% The program uses the OSGB implementation of the Redfearn series
% (see e.g. http://badc.nerc.ac.uk/help/coordinates/OSGB.pdf)
%
% Gareth Rees (wgr2@cam.ac.uk) & Zuzanna Swirad
% Scott Polar Research Institute
% University of Cambridge
% September 2012


switch proj
	case 'osgb'
		a=6377563.396;
		b=6356256.909;
		f0=0.9996012717;
		k0=49*pi/180;
		l0=-2*pi/180;
		e0=400000;
		n0=-100000;
	case 'Uwgs84'
		a=6378137;
		b=6356752.314;
		f0=0.9996;
		k0=0;
		l0=pi/180*(6*abs(zn)-183);
		e0=500000;
		n0=0;
	case 'Uint24'
		a=6378388;
		b=6356911.946;
		f0=0.9996;
		k0=0;
		l0=pi/180*(6*abs(zn)-183);
		e0=500000;
		n0=0;	
	case 'GKkras'
		a=6378245;
		b=6356863;
		f0=1;
		k0=0;
		l0=pi/180*(6*abs(zn)-183);
		e0=500000+(abs(zn)-30)*1000000;
		n0=0;	
	otherwise
		error('Unknown projection!')

end

if zn<0
	n0=10000000;
end

% convert input data to radians and bring longitude into range -180..+180

k=k*pi/180;
l=mod(l+180,360)-180;
l=l*pi/180;


a1=a*f0;
b1=b*f0;
e2=1-(b/a)^2;
n1=(a-b)/(a+b);

k3=k-k0;
k4=k+k0;

j3=(1+n1+5/4*n1^2+5/4*n1^3).*k3;
j4=(3*n1+3*n1^2+21/8*n1^3).*sin(k3).*cos(k4);
j5=(15/8*n1^2+15/8*n1^3).*sin(2*k3).*cos(2*k4);
j6=35/24*n1^3.*sin(3*k3).*cos(3*k4);
m=b1*(j3-j4+j5-j6);

v=a1./sqrt(1-e2*sin(k).^2);
r=v.*(1-e2)./(1-e2*sin(k).^2);
h2=v./r-1;

p=l-l0;
j3=m+n0;
j4=v/2.*sin(k).*cos(k);
j5=v/24.*sin(k).*cos(k).^3.*(5-tan(k).^2+9*h2);
j6=v/720.*sin(k).*cos(k).^5.*(61-58*tan(k).^2+tan(k).^4);
n=j3+p.^2.*j4+p.^4.*j5+p.^6.*j6;
j7=v.*cos(k);
j8=v/6.*cos(k).^3.*(v./r-tan(k).^2);
j9=v/120.*cos(k).^5;
j9=j9.*(5-18*tan(k).^2+tan(k).^4+14*h2-58*tan(k).^2.*h2);
e=e0+p.*j7+p.^3.*j8+p.^5.*j9;

end
