function [k l] = convert_from_utm(e, n, proj, zn)

% converts eastings (e) and northings (n)
% to latitude (k) and longitude (l), both in degrees, in a transverse Mercator
% projection. The variable 'proj' indicates the particular
% projection. If it is a UTM-type projection, the variable zn
% specifies the zone number. For Northern hemisphere zones, zn is
% a positive integer between 1 and 60. For Southern hemisphere zones,
% zn is a negative integer between -1 and -60. In either case, the longitude
% of the central meridian is 6*abs(zn)-183 degrees.
%
% More projections can be implemented using the 'switch' statement.
%
% Latitudes in the Southern hemisphere are negative.
% Longitudes in the Western hemisphere are negative.

% The parameters of the projection are
%   a: equatorial radius of ellipsoid
%   b: polar radius of ellipsoid
%   f0: scale factor on central meridian
%   k0: latitude of origin (radians)
%   l0: longitude of central meridian (radians)
%   e0: eastings coordinate of origin
%   n0: northings coordinate of meridian

% The program uses the OSGB implementation of the Redfearn series
% (see e.g. http://badc.nerc.ac.uk/help/coordinates/OSGB.pdf)
%
% Gareth Rees (wgr2@cam.ac.uk) & Zuzanna Swirad
% Scott Polar Research Institute
% University of Cambridge
% September 2012

switch proj
	case 'osgb'
		a=6377563.396;
		b=6356256.909;
		f0=0.9996012717;
		k0=49*pi/180;
		l0=-2*pi/180;
		e0=400000;
		n0=-100000;
	case 'Uwgs84'
		a=6378137;
		b=6356752.314;
		f0=0.9996;
		k0=0;
		l0=pi/180*(6*abs(zn)-183);
		e0=500000;
		n0=0;
	case 'Uint24'
		a=6378388;
		b=6356911.946;
		f0=0.9996;
		k0=0;
		l0=pi/180*(6*abs(zn)-183);
		e0=500000;
		n0=0;
	case 'GKkras'
		a=6378245;
		b=6356863;
		f0=1;
		k0=0;
		l0=pi/180*(6*abs(zn)-183);
		e0=500000+(abs(zn)-30)*1000000;
		n0=0;
	otherwise
		error('Unknown projection!')
end

if zn<0
	n0=10000000;
end

a1=a*f0;
b1=b*f0;
e2=1-(b/a)^2;
n1=(a-b)/(a+b);

k=(n-n0)./a1+k0;
m=0;
while (abs(n-n0-m)>0.001)
	k3=k-k0;
	k4=k+k0;
	j3=(1+n1+5/4*n1^2+5/4*n1^3)*k3;
	j4=(3*n1+3*n1^2+21/8*n1^3).*sin(k3).*cos(k4);
	j5=(15/8*n1^2+15/8*n1^3).*sin(2*k3).*cos(2*k4);
	j6=35/24*n1^3*sin(3*k3).*cos(3*k4);
	m=b1*(j3-j4+j5-j6);
	k=k+(n-n0-m)/a1;
end;

v=a1./sqrt(1-e2*sin(k).^2);
r=v.*(1-e2)./(1-e2*sin(k).^2);
h2=v./r-1;

y1=e-e0;
j3=tan(k)./(2*r.*v);
j4=tan(k)./(24*r.*v.^3).*(5+3*tan(k).^2+h2-9*tan(k).^2.*h2);
j5=tan(k)./(720*r.*v.^5).*(61+90*tan(k).^2+45*tan(k).^4);
k9=k-y1.^2.*j3+y1.^4.*j4-y1.^6.*j5;
j6=1./(cos(k).*v);
j7=1./(cos(k)*6.*v.^3).*(v./r+2*tan(k).^2);
j8=1./(cos(k)*120.*v.^5).*(5+28*tan(k).^2+24*tan(k).^4);
j9=1./(cos(k)*5040.*v.^7);
j9=j9.*(61+662*tan(k).^2+1320*tan(k).^4+720*tan(k).^6);
l=l0+y1.*j6-y1.^3.*j7+y1.^5.*j8-y1.^7.*j9;
k=k9;

l=l*180/pi;
l=mod(l+180,360)-180;
k=k*180/pi;

end